Package: ResidualMatrix
Version: 1.22.0
Date: 2025-04-08
Title: Creating a DelayedMatrix of Regression Residuals
Authors@R: person("Aaron", "Lun", role=c("aut", "cre", "cph"),
        email="infinite.monkeys.with.keyboards@gmail.com")
Imports: methods, Matrix, S4Vectors, DelayedArray
Suggests: testthat, BiocStyle, knitr, rmarkdown, BiocSingular
biocViews: Software, DataRepresentation, Regression, BatchEffect,
        ExperimentalDesign
Description: 
    Provides delayed computation of a matrix of residuals after fitting a
    linear model to each column of an input matrix. Also supports partial
    computation of residuals where selected factors are to be preserved in the
    output matrix. Implements a number of efficient methods for operating on
    the delayed matrix of residuals, most notably matrix multiplication 
    and calculation of row/column sums or means.
License: GPL-3
VignetteBuilder: knitr
RoxygenNote: 7.3.1
BugReports: https://github.com/LTLA/ResidualMatrix/issues
URL: https://github.com/LTLA/ResidualMatrix
git_url: https://git.bioconductor.org/packages/ResidualMatrix
git_branch: RELEASE_3_23
git_last_commit: bbbe857
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:11:08 UTC; biocbuild
Author: Aaron Lun [aut, cre, cph]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
