# Generated by roxygen2: do not edit by hand

export(ResidualMatrix)
export(ResidualMatrixSeed)
exportClasses(ResidualMatrix)
exportClasses(ResidualMatrixSeed)
exportMethods("%*%")
exportMethods("[")
exportMethods("dimnames<-")
exportMethods(DelayedArray)
exportMethods(colMeans)
exportMethods(colSums)
exportMethods(crossprod)
exportMethods(dim)
exportMethods(dimnames)
exportMethods(extract_array)
exportMethods(rowMeans)
exportMethods(rowSums)
exportMethods(show)
exportMethods(t)
exportMethods(tcrossprod)
import(methods)
importClassesFrom(DelayedArray,DelayedMatrix)
importFrom(DelayedArray,DelayedArray)
importFrom(DelayedArray,extract_array)
importFrom(DelayedArray,new_DelayedArray)
importFrom(DelayedArray,seed)
importFrom(Matrix,colMeans)
importFrom(Matrix,colSums)
importFrom(Matrix,crossprod)
importFrom(Matrix,drop)
importFrom(Matrix,rowMeans)
importFrom(Matrix,rowSums)
importFrom(Matrix,t)
importFrom(Matrix,tcrossprod)
importFrom(S4Vectors,setValidity2)
