# DelayedMatrix of regression residuals

|Build|Status|
|-----|----|
| [Bioc-release](https://bioconductor.org/packages/release/bioc/html/ResidualMatrix.html) | [![](https://bioconductor.org/shields/build/release/bioc/ResidualMatrix.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/ResidualMatrix) |
| [Bioc-devel](https://bioconductor.org/packages/devel/bioc/html/ResidualMatrix.html) | [![](https://bioconductor.org/shields/build/devel/bioc/ResidualMatrix.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/ResidualMatrix) | 

This package performs the delayed calculation of a matrix of residuals after fitting a linear model to each column of an input matrix.
It implements a number of efficient methods for operating on the delayed matrix of residuals, most notably matrix multiplication and calculation of row/column sums or means.
Check out the user's guide on the [Bioconductor landing page](https://bioconductor.org/packages/devel/bioc/html/ResidualMatrix.html) for more details.
